; routine to trim leading zeros from a string.
; B = char to check.
; Routine terminates first <> "0" char it runs into.

trim_zeros	ld	a,'0'		; ASCII zero.
		cp	(hl)		; HL points to buffer under test.
		ret	nz
		ld	(hl),NULL.asc	; SPACE Stuff it.
		inc	hl
		jr	trim_zeros

; Basic 8-bit Square Root
; Integer squares can be generated iteratively by adding consecutive odd numbers.
; Simplest way to calculate square root is a reverse of this process
; take a number and subtract successive odd integers from it.
; Number of iterations it takes until the result is negative is the radix.

; Input: A = Radicand
; Output: B = Radix


Sqrt8		ld	b,-1
Sqrt8_1		inc	b
		inc	b
		sub	b
		jr	nc,Sqrt8_1	; use JP to save 2 T
		srl	b		; translate (2X + 1) to X
		ret

cvt8_hex2ascii	push	af		; Save binary copy for later.
		and	F0h		; Strip all but upper 4 bits.
		srl	a		; Shift right.
		srl	a
		srl	a
		srl	a
		call	cvt2asc
		pop	af		; Recover original values.
		and	0Fh		; Strip out upper 4 bits.			
cvt2asc		add	'0'		; Convert nibble to ASCII.
		ld	(hl),a		; Store it in buffer.
		inc	hl		; Bump pointer up by 1.
		ret

; Hexadecimal conversion operates directly on nibbles and takes advantage of nifty DAA trick.
; Input: HL = number to convert, DE = location of ASCII string
; Output: ASCII string at (DE)

cvt16_bin2hex	ld	a,h
		call	Num2Hex1
		ld	a,h
		call	Num2Hex2
		ld	a,l
		call	Num2Hex1
		ld	a,l
		call	Num2Hex2
Num2Hex1	rra
		rra
		rra
		rra
Num2Hex2	or	F0h
		daa
		add	a,A0h
		adc	a,40h
		ld	(de),a
		inc	de
		ret

cvt24_bin2ascii	macro

; Routine for converting a 24-bit binary number to decimal
; In: E:HL = 24-bit binary number (0-16777215)
; Out: DE:HL = 8 digit decimal form (packed BCD)
; Changes: AF, BC, DE, HL & IX


; Straight from my time-tested toolbox Smile If you want bigger numbers: let me know.
; I also have a ; routine that can do up to 64 bit numbers (20 digits decimal) and produces ready-for-printing result, 
; but it's quite a bit longer. Probably did it once just to see how easy it was to expand, or to have 
; something that can take anything you throw at it...
; by Alwin Henseler

		 LD 	C,E
                 PUSH 	HL
                 POP 	IX          	; input value in C:IX
                 LD 	HL,1
                 LD 	D,H
                 LD 	E,H          	; start value corresponding with 1st 1-bit
                 LD 	B,24         	; bitnr. being processed + 1

FIND1:           ADD 	IX,IX
                 RL 	C            	; shift bit 23-0 from C:IX into carry
                 JR 	C,NEXTBIT
                 DJNZ 	FIND1      	; find highest 1-bit

; All bits 0:
                 RES 	0,L         	; least significant bit not 1 after all ..
                 RET

DBLLOOP:         LD 	A,L
                 ADD 	A,A
                 DAA
                 LD 	L,A
                 LD 	A,H
                 ADC 	A,A
                 DAA
                 LD 	H,A
                 LD 	A,E
                 ADC 	A,A
                 DAA
                 LD 	E,A
                 LD 	A,D
                 ADC 	A,A
                 DAA
                 LD 	D,A        	; double the value found so far
                 ADD 	IX,IX
                 RL 	C            	; shift next bit from C:IX into carry
                 JR 	NC,NEXTBIT   	; bit = 0 -> don't add 1 to the number
                 SET 	0,L         	; bit = 1 -> add 1 to the number
NEXTBIT:         DJNZ 	DBLLOOP
                 RET

		endmac	cvt24_bin2ascii